function dtheta_fixed = calc_poincare_fp
%CALC_POINCARE_FP  Calculate the Poincare map's fixed point.

%Copyright (c) 2003 by Eric Westervelt and Jessy Grizzle.  This code
%may be freely used for noncommercial ends. If use of this code in
%part or in whole results in publication, proper citation must be
%included in that publication.  This code comes with no guarantees or
%support.

%Eric R. Westervelt & Jessy W. Grizzle
%8/6/01
%11/30/03 - updated for the CDC

% Calculate constants of explicit form of Poincare map (p = z2^-)
%
%   p(k+1) = (C1*p(k)+C2)^(1/2)
%
% where
%
%   C1 = impact map (its linear!!)
%
% and
%
%   C2 = int(beta(z1)/alpha(z1),z1^-,z1^+)
%
[a,b,m] = controlParameters;

z1 = linspace(b,b+m,1000);
alpha_times_beta = alpha_beta(z1);
  
C2_cum = cumtrapz(z1,alpha_times_beta);

K = min(C2_cum);
disp(['[K = ',num2str(K),']']);

C2 = trapz(z1,alpha_times_beta);

x_minus = sigma(1);
q_minus = x_minus(1:5);

x_plus = impact_lap_red(x_minus);
D = dyn_mod_lap_red(x_plus(1:5),zeros(5,1));
D_N = D(5,:);
C1 = D_N*x_plus(6:10);

z2_minus_fixed_point = -sqrt(2*C2/(1-C1^2));

z_minus = [b + m; z2_minus_fixed_point];
x_minus = zd_s_lift(z_minus);

dtheta_fixed = dtheta_s(x_minus);
