function symb_model_lap
%SYMB_MODEL_LAP

%Copyright (c) 2003 by Eric Westervelt and Jessy Grizzle.  This code
%may be freely used for noncommercial ends. If use of this code in
%part or in whole results in publication, proper citation must be
%included in that publication.  This code comes with no guarantees or
%support.

%Eric R. Westervelt & Jessy W. Grizzle
%11/30/03 - updated for the CDC

if isempty(dir('mat_files_sym/work_symb_model_lap.mat'))
  % This is for a robot with an upright trunk, two legs and knees. The
  % model is for full seven degrees of freedom of the robot
  %
  syms q31L q32L q41L q42L q1L real
  syms dq31L dq32L dq41L dq42L dq1L real
  syms y z dy dz ycm zcm dycm dzcm real
  syms g L1 L3 L4 M1 M3 M4 real
  syms MY1 MY3 MY4 MZ1 MZ3 MZ4 XX1 XX3 XX4 IA3 IA4 real
  syms FV1 FV2 FV3 FV4 FS1 FS2 FS3 FS4 real
  
  syms Lb1 Lt XXb XXc XXw PEbc PEcc PEwc real
  % symb Lb Mb Mc Mw real
  %
  % Change coordinates
  %
  [qlap,dqlap,q_b,dq_b] = abs2lapin;
  %
  % old absolute coordinates in terms of relative coords
  q31 = q_b(1);
  q32 = q_b(2);
  q41 = q_b(3);
  q42 = q_b(4);
  q1 =  q_b(5);
  dq31 = dq_b(1);
  dq32 = dq_b(2);
  dq41 = dq_b(3);
  dq42 = dq_b(4);
  dq1 =  dq_b(5);
  q =  qlap;
  dq = dqlap;
  %
  % reference = absolute angles for everything; trigonometric sense
  %             for all angles, measured from the vertical,
  %
  q  = [q; ycm; zcm];
  q_old = [q(1:5); y; z];
  dq = [dq; dycm; dzcm];
  dq_old = [dq(1:5); dy; dz];
  %
  % centers of gravity and positions of the various members 
  %
  % The rotation matrix, R(t), rotates the vector [MYx; MZx]/Mx (which
  % originates at the joint and ends at the link center of mass) an
  % angle t in the (Y-Z) plane (X points our of the plane).
  %
  % see computing_the_Lagrangian.pdf
  %
  R = inline('[cos(t) -sin(t); sin(t) cos(t)]');
  pH    = [y; z];
  pT    = pH + R(q1)*[MY1; MZ1]/M1;
  pFem1 = pH + R(q31)*[MY3; MZ3]/M3;
  pFem2 = pH + R(q32)*[MY3; MZ3]/M3;
  pG1   = pH + R(q31)*[0;1]*L3;
  pG2   = pH + R(q32)*[0;1]*L3;
  pTib1 = pG1 + R(q41)*[MY4; MZ4]/M4;
  pTib2 = pG2 + R(q42)*[MY4; MZ4]/M4;
  %
  %
  M_total = M1 + 2*M3 + 2*M4;
  cm = simple(1/M_total*(M1*pT+M3*pFem1+M3*pFem2+M4*pTib1+M4*pTib2));
  dcm = simple(jacobian(cm,q_old)*dq_old);
  %
  y = solve(cm(1)-ycm,'y'); y = simple(y);
  z = solve(cm(2)-zcm,'z'); z = simple(z);
  dy = solve(dcm(1)-dycm,'dy'); dy = simple(dy);
  dz = solve(dcm(2)-dzcm,'dz'); dz = simple(dz);
  %
  pH = [y; z];
  pT    = pH + R(q1)*[MY1; MZ1]/M1;
  pFem1 = pH + R(q31)*[MY3; MZ3]/M3;
  pFem2 = pH + R(q32)*[MY3; MZ3]/M3;
  pG1   = pH + R(q31)*[0;1]*L3;
  pG2   = pH + R(q32)*[0;1]*L3;
  pTib1 = pG1 + R(q41)*[MY4; MZ4]/M4;
  pTib2 = pG2 + R(q42)*[MY4; MZ4]/M4;
  pFoot1= pG1 + R(q41)*[0;1]*L4;
  pFoot2= pG2 + R(q42)*[0;1]*L4;
  pHead = pH +  R(q1)*[0;1]*L1;
  %
  %
  vH =  jacobian(pH,q)*dq;
  vT =  jacobian(pT,q)*dq;
  vG1 = jacobian(pG1,q)*dq;
  vG2 = jacobian(pG2,q)*dq;
  %
  %
  vH_RT =    R(q1).'*vH;
  vH_Fem1 =  R(q31).'*vH;
  vH_Fem2 =  R(q32).'*vH;
  vG1_Tib1 = R(q41).'*vG1;
  vG2_Tib2 = R(q42).'*vG2;
  %
  %
  KET    = simplify(1/2*M1*vH.'*vH ...
		    +vH_RT.'*[-MZ1;MY1]*dq1 ...
		    +1/2*XX1*dq1^2);
  % actuator interia only depends on the relative link angle (see
  % the IAx term)
  KEFem1 = simplify(1/2*M3*vH.'*vH ...
		    +vH_Fem1.'*[-MZ3;MY3]*(dq31) ...
		    +1/2*XX3*dq31^2 ...
		    +1/2*IA3*(dq31-dq1)^2); 
  KEFem2 = simplify(1/2*M3*vH.'*vH ...
		    +vH_Fem2.'*[-MZ3;MY3]*(dq32) ...
		    +1/2*XX3*dq32^2 ...
		    +1/2*IA3*(dq32-dq1)^2);
  KETib1 = simplify(1/2*M4*vG1.'*vG1 ...
		    +vG1_Tib1.'*[-MZ4;MY4]*(dq41) ...
		    +1/2*XX4*dq41^2 ...
		    +1/2*IA4*(dq41-dq31)^2);
  KETib2 = simplify(1/2*M4*vG2.'*vG2 ...
		    +vG2_Tib2.'*[-MZ4;MY4]*(dq42) ...
		    +1/2*XX4*dq42^2 ...
		    +1/2*IA4*(dq42-dq32)^2);
  %
  % Including the tower/bar interia
  %
  % horizontal interia
  if 0
    phi_rot_h = pH(1)/Lb1;
  else
    % an approximation
    phi_rot_h = ((-q31L-q41L+pi)*L3 + (-q31L-q41L-q1L+pi)*L4)/Lb1;
  end
  dphi_rot_h = jacobian(phi_rot_h,q)*dq;
  %
  KEbar_h = XXb*dphi_rot_h^2;
  KEcables_h = XXc*dphi_rot_h^2;
  KEweights_h = XXw*dphi_rot_h^2;
  %
  % vertical inertia
  phi_rot_v = (pH(2)-Lt)/Lb1;
  sin_phi_rot_v = phi_rot_v;
  %
  if 0
    dphi_rot_v = jacobian(phi_rot_v,q)*dq;
    KEbar_v = 1/2*Mb/Lb*(Lb1^3/3 + Lb2^3/3)*dphi_rot_v^2;
    KEcables_v = 1/2*Mc/Lb1*(Lb1^3/3)*dphi_rot_v^2;
    KEweights_v = 1/2*Mw*(Lb2^2)*dphi_rot_v^2;
    %
    KE = simplify(KET + KEFem1 + KEFem2 + KETib1 + KETib2 + ...
		  KEbar_h + KEcables_h + KEweights_h + ...
		  KEbar_v + KEcables_v + KEweights_v);
  else
    % omit "vertical" inertia for now
    KE = simplify(KET + KEFem1 + KEFem2 + KETib1 + KETib2 + ...
		  KEbar_h + KEcables_h + KEweights_h);
    KE = simple(KE);
  end
  %
  %
  vFoot1 = jacobian(pFoot1,q)*dq;
  vFoot2 = jacobian(pFoot2,q)*dq;
  %
  % Including the tower/bar interia
  %
  PEbar = PEbc*sin_phi_rot_v;
  PEcables = PEcc*sin_phi_rot_v;
  PEweights = PEwc*sin_phi_rot_v;
  %
  PE = g*(M1*pT(2) + M3*pFem1(2) + M3*pFem2(2) + M4*pTib1(2) + ...
	  M4*pTib2(2) + ...
	  PEbar + PEcables + PEweights);
  PE = simple(PE);
  %
  %
  % Model NOTATION: Spong and Vidyasagar, page 142, Eq. (6.3.12)
  %                 D(q)ddq + C(q,dq)*dq + G(q) = B*tau + E1*F_external
  %
  %L=KE-PE;
  G = jacobian(PE,q).';
  G = simple(G);
  D = jacobian(KE,dq).';
  n = max(size(q));
  for k = 1:n
    D_tmp = D(k);
    tic
    D_tmp2 = simple(D_tmp);
    toc
    D(k) = D_tmp2;
    disp(['D simple ',num2str(k)]);
  end
  D = jacobian(D,dq);
  for k = 1:n
    for j = 1:k
      D(k,j) = simple(D(k,j));
      disp(['D simple (',num2str(k),',',num2str(j),')']);
    end
    D(j,k) = D(k,j);
  end
  %
  syms C real
  for k = 1:n
    for j = 1:k
      C(k,j) = 0*g;
      for i = 1:n
	C(k,j) = C(k,j) + 1/2*(diff(D(k,j),q(i)) ...
			       + diff(D(k,i),q(j)) ...
			       - diff(D(i,j),q(k)))*dq(i);
      end
    end
    disp(['C simple (',num2str(k),',',num2str(j),')']);
    C(k,j) = simple(C(k,j));
    C(j,k) = C(k,j);
  end
  % C = simple(C);
  %
  % Compute the matrix for the input torques and then the contact
  % points.
  %
  Phi_0 = [q31-q1; q32-q1; q41-q31; q42-q32; q1; y; z];
  B = jacobian(Phi_0,q);
  B = B.'*[eye(4,4); zeros(3,4)];
  %
  %
  % F_ext = [F_T^1;F_N^1;F_T^2;F_N^2];
  %
  Phi_1 = [pFoot1; pFoot2];
  E1 = jacobian(Phi_1,q).';
  %
  % Compute matrices associated with the viscous and static
  % friction
  %
  EFV=diag([FV1,FV2,FV3,FV4,0,0,0])*dq;
  EFS=diag([FS1,FS2,FS3,FS4,0,0,0])*2/pi*atan(dq*5);
  %
  % Now, compute the forces acting on the support foot, assuming that
  % the robot is in the swing phase
  %
  c = pFoot1;
  jac_c = jacobian(c,q);             % c_dot  = jac_c * dq
  lfc = jac_c*dq;
  g1 = jac_c.';
  gamma_c = jacobian(jac_c*dq,q)*dq; % c_ddot = gamma_c + jac_c * ddq
  
  save mat_files_sym/work_symb_model_lap
  disp('[DONE saving MAT_FILES_SYM/WORK_SYMB_MODEL_LAP.MAT]')
else
  load mat_files_sym/work_symb_model_lap
  disp('[DONE loading MAT_FILES_SYM/WORK_SYMB_MODEL_LAP.MAT]')
end

fid = fopen('m_files/modelP.m','w');
print_modelParameters(fid);
fclose(fid);

fcn_name = 'dyn_mod_lap';
print_model
%
%
%
fid=print_function_header('stance_foot_vel',...
			  'vh,vv',...
			  'x',...
			  ['']);

fprintf(fid,'modelP;\n\n');

fprintf(fid,['q31L=x(1);q32L=x(2);q41L=x(3);q42L=x(4);' ...
	     'q1L=x(5); ycm=x(6);zcm=x(7);\n']);
fprintf(fid,['dq31L=x(8);dq32L=x(9);dq41L=x(10);dq42L=x(11);' ...
	     'dq1L=x(12); dycm=x(13);dzcm=x(14);\n\n']);

ttt = char(vFoot1(1));
fprintf(fid,'vh = %s;\n',fixlength(ttt,'*+-',65,'    '));
ttt = char(vFoot1(2));
fprintf(fid,'vv = %s;\n',fixlength(ttt,'*+-',65,'    '));
fclose(fid);
