function symb_stance_force
%SYMB_STANCE_FORCE.M

%Copyright (c) 2003 by Eric Westervelt and Jessy Grizzle.  This code
%may be freely used for noncommercial ends. If use of this code in
%part or in whole results in publication, proper citation must be
%included in that publication.  This code comes with no guarantees or
%support.

%Eric R. Westervelt & Jessy W. Grizzle
%7/12/01
%11/30/03 - updated for the CDC

if isempty(dir('mat_files_sym/work_symb_stance_force.mat'))
  % This is for a robot with an upright trunk, two legs and
  % knees. The model is for five degrees of freedom of the robot,
  % with Foot1 touching the ground
  %
  syms q31L q32L q41L q42L q1L real
  syms dq31L dq32L dq41L dq42L dq1L real
  syms ddq31L ddq32L ddq41L ddq42L ddq1L real
  syms y z dy dz ddy ddz real
  syms g L1 L3 L4 M1 M3 M4 real
  syms MY1 MY3 MY4 MZ1 MZ3 MZ4 XX1 XX3 XX4 IA3 IA4 real
  
  syms Lb1 Lt XXb XXc XXw PEbc PEcc PEwc real
  %
  % Change coordinates
  %
  [qlap,dqlap,q_b,dq_b] = abs2lapin;
  %
  % old absolute coordinates in terms of lapin coordinates
  q31 = q_b(1);
  q32 = q_b(2);
  q41 = q_b(3);
  q42 = q_b(4);
  q1 =  q_b(5);
  dq31 = dq_b(1);
  dq32 = dq_b(2);
  dq41 = dq_b(3);
  dq42 = dq_b(4);
  dq1 =  dq_b(5);
  q =  qlap;
  dq = dqlap;
  %
  % reference = absolute angles for everything; trigonometric sense for all
  %              angles, measured from the vertical,
  %
  q = [q; y; z];
  dq = [dq; dy; dz];
  ddq = [ddq31L; ddq32L; ddq41L; ddq42L; ddq1L; ddy; ddz];
  %
  % 
  % The rotation matrix, R(t), rotates the vector [MYx; MZx]/Mx (which
  % originates at the joint and ends at the link center of mass) an
  % angle t in the (Y-Z) plane (X points our of the plane).
  %
  R = inline('[cos(t) -sin(t); sin(t) cos(t)]');
  pH = [y;z]-(R(q31)*[0;1]*L3 + R(q41)*[0;1]*L4);
  pG1 = pH + R(q31)*[0;1]*L3;
  pG2 = pH + R(q32)*[0;1]*L3;
  %
  %
  vH =  jacobian(pH,q)*dq;
  vG1 = jacobian(pG1,q)*dq;
  vG2 = jacobian(pG2,q)*dq;
  %
  %
  vH_RT =    R(q1).'*vH;
  vH_Fem1 =  R(q31).'*vH;
  vH_Fem2 =  R(q32).'*vH;
  vG1_Tib1 = R(q41).'*vG1;
  vG2_Tib2 = R(q42).'*vG2;
  %
  %
  KET    = simplify(1/2*M1*vH.'*vH ...
		    +vH_RT.'*[-MZ1;MY1]*dq1 ...
		    +1/2*XX1*dq1^2);
  % actuator interia only depends on the relative link angle (see
  % the IAx term)
  KEFem1 = simplify(1/2*M3*vH.'*vH ...
		    +vH_Fem1.'*[-MZ3;MY3]*(dq31) ...
		    +1/2*XX3*dq31^2 ...
		    +1/2*IA3*(dq31-dq1)^2); 
  KEFem2 = simplify(1/2*M3*vH.'*vH ...
		    +vH_Fem2.'*[-MZ3;MY3]*(dq32) ...
		    +1/2*XX3*dq32^2 ...
		    +1/2*IA3*(dq32-dq1)^2);
  KETib1 = simplify(1/2*M4*vG1.'*vG1 ...
		    +vG1_Tib1.'*[-MZ4;MY4]*(dq41) ...
		    +1/2*XX4*dq41^2 ...
		    +1/2*IA4*(dq41-dq31)^2);
  KETib2 = simplify(1/2*M4*vG2.'*vG2 ...
		    +vG2_Tib2.'*[-MZ4;MY4]*(dq42) ...
		    +1/2*XX4*dq42^2 ...
		    +1/2*IA4*(dq42-dq32)^2);
  %
  % Including the tower/bar interia
  %
  %Lb2 = Lb - Lb1;
  %
  % horizontal interia
  if 0
    phi_rot_h = pH(1)/Lb1;
  else
    % an approximation
    phi_rot_h = ((-q31L-q41L+pi)*L3 + (-q31L-q41L-q1L+pi)*L4)/Lb1;
  end
  dphi_rot_h = jacobian(phi_rot_h,q)*dq;
  %
  KEbar_h = XXb*dphi_rot_h^2;
  KEcables_h = XXc*dphi_rot_h^2;
  KEweights_h = XXw*dphi_rot_h^2;
  %  KEbar_h = 1/2*Mb/Lb*(Lb1^3/3 + Lb2^3/3)*dphi_rot_h^2;
  %  KEcables_h = 1/2*Mc/Lb1*(Lb1^3/3)*dphi_rot_h^2;
  %  KEweights_h = 1/2*Mw*(Lb2^2)*dphi_rot_h^2;
  %
  % vertical inertia
  phi_rot_v = (pH(2)-Lt)/Lb1;
  sin_phi_rot_v = phi_rot_v;
  dphi_rot_v = jacobian(phi_rot_v,q)*dq;
  %
  if 0
    KEbar_v = 1/2*Mb/Lb*(Lb1^3/3 + Lb2^3/3)*dphi_rot_v^2;
    KEcables_v = 1/2*Mc/Lb1*(Lb1^3/3)*dphi_rot_v^2;
    KEweights_v = 1/2*Mw*(Lb2^2)*dphi_rot_v^2;
    %
    KE = simplify(KET + KEFem1 + KEFem2 + KETib1 + KETib2 + ...
		  KEbar_h + KEcables_h + KEweights_h + ...
		  KEbar_v + KEcables_v + KEweights_v);
  else
    % omit "vertical" inertia for now
    KE = simplify(KET + KEFem1 + KEFem2 + KETib1 + KETib2 + ...
		  KEbar_h + KEcables_h + KEweights_h);
    KE=simple(KE);
  end
  %
  %
  % centers of gravity and positions of the various members 
  %
  %pT = pH + ...
  %(1/M1)*(MY1^2+MZ1^2)^(.5)*[-sin(q1-atan(MY1/MZ1));cos(q1-atan(MY1/MZ1))];
  pT    = pH + R(q1)*[MY1; MZ1]/M1;
  pFem1 = pH + R(q31)*[MY3; MZ3]/M3;
  pFem2 = pH + R(q32)*[MY3; MZ3]/M3;
  pG1   = pH + R(q31)*[0; 1]*L3;
  pG2   = pH + R(q32)*[0; 1]*L3;
  pTib1 = pG1 + R(q41)*[MY4; MZ4]/M4;
  pTib2 = pG2 + R(q42)*[MY4; MZ4]/M4;
  pFoot1= pG1 + R(q41)*[0; 1]*L4;
  pFoot2= pG2 + R(q42)*[0; 1]*L4;
  %
  % Including the tower/bar interia
  %
  PEbar = PEbc*sin_phi_rot_v;
  PEcables = PEcc*sin_phi_rot_v;
  PEweights = PEwc*sin_phi_rot_v;
  %  PEbar = g/2*Mb/Lb*sin_phi_rot_v*(Lb1^2 - Lb2^2);
  %  PEcables = g/2*Mc/Lb1*sin_phi_rot_v*Lb1^2;
  %  PEweights = -g/2*Mw*sin_phi_rot_v*Lb2;
  %
  PE = g*(M1*pT(2) + M3*pFem1(2) + M3*pFem2(2) + M4*pTib1(2) + ...
	  M4*pTib2(2) + ...
	  PEbar + PEcables + PEweights);
  PE = simple(PE);
  %
  %
  % Model NOTATION: Spong and Vidyasagar, page 142, Eq. (6.3.12)
  %                 D(q)ddq + C(q,dq)*dq + G(q) = B*tau + E2*F_external
  %
  %L = KE-PE;
  G = jacobian(PE,q).';
  G = simple(G);
  D = simple(jacobian(KE,dq).');
  D = simple(jacobian(D,dq));
  %
  syms C real
  n = max(size(q));
  for k = 1:n
    for j = 1:n
      C(k,j) = 0*g;
      for i = 1:n
	C(k,j) = C(k,j)+1/2*(diff(D(k,j),q(i)) + ...
			     diff(D(k,i),q(j))- ...
			     diff(D(i,j),q(k)))*dq(i);
      end
    end
  end
  C = simple(C);
  %
  for k = 1:n
    for j = 1:n
      for i = 1:n
	cc(i,j,k) = 1/2*(diff(D(k,j),q(i)) + ...
			 diff(D(k,i),q(j)) - ...
			 diff(D(i,j),q(k)));
      end
    end
  end
  %
  % Compute the matrix for the input torques and then the contact
  % points.
  %
  Phi_0 = [q31-q1;q32-q1;q41-q31;q42-q32;q1;y;z];
  B = jacobian(Phi_0,q);
  B = B.'*[eye(4,4);zeros(3,4)];
  %
  % Compute matrices associated with the viscous and static
  % friction
  %
  D21 = D(6:7,1:5);
  C21 = C(6:7,1:5);
  G2  = G(6:7);
  %
  % See my notes, 7/13/01 for equations...
  %
  F = simple(D21*ddq(1:5) + C21*dq(1:5) + G2);
  %
  save mat_files_sym/work_symb_stance_force
else
  load mat_files_sym/work_symb_stance_force
  disp('[DONE loading mat_files_sym/work_symb_stance_force.mat]')
end
%
%
%
%% write file header
%
fid=print_function_header('stance_force',...
			  'f_tan,f_norm',...
			  'x,ddq',...
			  ['']);

%% Read in constants
%
fprintf(fid,'modelP;\n\n');

fprintf(fid,'%% tangential force\n');
ttt = char(F(1));
ttt = rep_str(ttt,2);
ttt = fixlength(ttt,'*+-',65,'        ');
fprintf(fid,'f_tan = %s;\n\n',ttt);

fprintf(fid,'%% normal force\n');
ttt = char(F(2));
ttt = rep_str(ttt,2);
ttt = fixlength(ttt,'*+-',65,'         ');
fprintf(fid,'f_norm = %s;',ttt);
fclose(fid);

%------------------------------------------------------

function ttt_out = rep_str(ttt,type)

if type ==1
  ttt = strrep(ttt,'ddq31L','ddq(:,1)');
  ttt = strrep(ttt,'ddq32L','ddq(:,2)');
  ttt = strrep(ttt,'ddq41L','ddq(:,3)');
  ttt = strrep(ttt,'ddq42L','ddq(:,4)');
  ttt = strrep(ttt,'ddq1L', 'ddq(:,5)');
  ttt = strrep(ttt,'dq31L','x(:,6)');
  ttt = strrep(ttt,'dq32L','x(:,7)');
  ttt = strrep(ttt,'dq41L','x(:,8)');
  ttt = strrep(ttt,'dq42L','x(:,9)');
  ttt = strrep(ttt,'dq1L', 'x(:,10)');
  ttt = strrep(ttt,'q31L','x(:,1)');
  ttt = strrep(ttt,'q32L','x(:,2)');
  ttt = strrep(ttt,'q41L','x(:,3)');
  ttt = strrep(ttt,'q42L','x(:,4)');
  ttt = strrep(ttt,'q1L', 'x(:,5)');
  ttt=vectorize(ttt);
else
  ttt = strrep(ttt,'ddq31L','ddq(1)');
  ttt = strrep(ttt,'ddq32L','ddq(2)');
  ttt = strrep(ttt,'ddq41L','ddq(3)');
  ttt = strrep(ttt,'ddq42L','ddq(4)');
  ttt = strrep(ttt,'ddq1L', 'ddq(5)');
  ttt = strrep(ttt,'dq31L','x(6)');
  ttt = strrep(ttt,'dq32L','x(7)');
  ttt = strrep(ttt,'dq41L','x(8)');
  ttt = strrep(ttt,'dq42L','x(9)');
  ttt = strrep(ttt,'dq1L', 'x(10)');
  ttt = strrep(ttt,'q31L','x(1)');
  ttt = strrep(ttt,'q32L','x(2)');
  ttt = strrep(ttt,'q41L','x(3)');
  ttt = strrep(ttt,'q42L','x(4)');
  ttt = strrep(ttt,'q1L', 'x(5)');
end

ttt_out = ttt;
